#include <bits/stdc++.h>
using namespace std;
typedef long long s64;

template<typename T>
T epow(T a, T n){
    if(n == 0){
        return 1;
    }
    else{
        T z = epow(a, n / 2);
        z *= z;
        return n % 2 == 0 ? z : a * z;
    }
}

template<typename T>
T gcd(T a, T b){
    if(a > b){
        swap(a, b);
    }
    while(a != 0){
        b %= a;
        swap(a, b);
    }
    return b;
}

struct dot{
    double x, y;

    dot(){
        x = y = 0;
    }
    dot(double _x, double _y){
         x = _x;
         y = _y;
    }

    dot operator + (const dot& k) const{
        return dot(x + k.x, y + k.y);
    }
    dot operator - (const dot& k) const{
        return dot(x - k.x, y - k.y);
    }

    double getLen() const{
        return sqrt(x * x + y * y);
    }

    bool operator < (const dot& Rhs) const{
        return x == Rhs.x ? y < Rhs.y : x < Rhs.x;
    }

    double operator % (const dot& p) const{
        return x * p.y - y * p.x;
    }
};

vector<dot> buildConvexHull(vector<dot> Dots){
    sort(Dots.begin(), Dots.end());
    dot start = Dots[0];
    for(auto &p : Dots){
        p = p - start;
    }
    sort(Dots.begin() + 1, Dots.end(), [](const dot& Lhs, const dot& Rhs){
        double s = Lhs % Rhs;
        if(s == 0){
            return Lhs.getLen() < Rhs.getLen();
        }
        else{
            return s < 0;
        }
    });

    vector<dot> res;
    for(auto &p : Dots){
        while(res.size() > 1 && (p - res[res.size() - 2]) % (res.back() - res[res.size() - 2]) <= 0){
            res.pop_back();
        }
        res.push_back(p);
    }
    for(auto &p : res){
        p = p + start;
    }
    return res;
}

double getPerim(const vector<dot>& Dots){
    double ans = 0;
    for(int i = 0; i < Dots.size(); i++){
        dot p1 = Dots[i];
        dot p2 = i ? Dots[i - 1] : Dots.back();
        ans += (p1 - p2).getLen();
    }
    return ans;
}

int main() {
    //ifstream cin("input.txt");
    ios::sync_with_stdio(0), cin.tie(0), cout.tie(0);

    int n;
    double l;
    cin >> n >> l;
    set<dot> S;
    for(int i = 0; i < n; i++){
        dot p;
        cin >> p.x >> p.y;
        S.insert(p);
        S.insert(dot(p.x, p.y + l));
        S.insert(dot(p.x, p.y - l));
        S.insert(dot(p.x - l, p.y));
        S.insert(dot(p.x + l, p.y));
    }

    auto it = S.begin();
    vector<dot> Dots(S.size());
    for(int i = 0; i < S.size(); i++){
        Dots[i] = *it;
        it++;
    }

    cout << fixed() << setprecision(6);
    cout << getPerim(buildConvexHull(Dots)) << "\n";



    return 0;
}
