#include <iostream>
#include <string>
#include <vector>
#include <algorithm>
#include <cmath>
#include <set>
#include <iomanip>
#include <deque>

using namespace std;

#define int long long

struct str {
    double x, y;
};

bool cmp(str a, str b) {
    return a.x < b.x || a.x == b.x && a.y < b.y;
}

bool cw(str a, str b, str c) {
    return a.x * (b.y - c.y) + b.x * (c.y - a.y) + c.x * (a.y - b.y) < 0;
}

bool ccw(str a, str b, str c) {
    return a.x * (b.y - c.y) + b.x * (c.y - a.y) + c.x * (a.y - b.y) > 0;
}

void convex_hull(vector<str> &a) {
    if (a.size() == 1) return;
    sort(a.begin(), a.end(), &cmp);
    str p1 = a[0], p2 = a.back();
    vector<str> up, down;
    up.push_back(p1);
    down.push_back(p1);
    for (int i = 1; i < a.size(); ++i) {
        if (i == a.size() - 1 || cw(p1, a[i], p2)) {
            while (up.size() >= 2 && !cw(up[up.size() - 2], up[up.size() - 1], a[i]))
                up.pop_back();
            up.push_back(a[i]);
        }
        if (i == a.size() - 1 || ccw(p1, a[i], p2)) {
            while (down.size() >= 2 && !ccw(down[down.size() - 2], down[down.size() - 1], a[i]))
                down.pop_back();
            down.push_back(a[i]);
        }
    }
    a.clear();
    for(int i = 0; i < up.size(); ++i)
        a.push_back(up[i]);
    for(int i = down.size() - 2; i > 0; --i)
        a.push_back(down[i]);
}

double len(double x1, double y1, double x2, double y2){
    return sqrt((x2 - x1) * (x2 - x1) + (y2 - y1) * (y2 - y1));
}

void solve() {
    int n;
    double l;
    cin >> n >> l;
    vector<str> v(n);
    for (int i = 0; i < n; i++) {
        cin >> v[i].x >> v[i].y;
    }
    vector<str> pt;
    for (int i = 0; i < n; i++) {
        pt.push_back({v[i].x + l, v[i].y});
        pt.push_back({v[i].x - l, v[i].y});
        pt.push_back({v[i].x, v[i].y + l});
        pt.push_back({v[i].x, v[i].y - l});

    }
    convex_hull(pt);
    double ans = 0;
    for(int i = 0; i < pt.size() - 1; i++){
        ans += len(pt[i].x, pt[i].y, pt[i + 1].x, pt[i + 1].y);
    }
    ans += len(pt[pt.size() - 1].x, pt[pt.size() - 1].y, pt[0].x, pt[0].y);
    cout << setprecision(8) << fixed << ans;
}

signed main() {
    solve();
    return 0;
}
/*
4 3
-3 -3
-3 3
3 3
3 -3

9 0
1 1
3 1
5 1
5 2
3 2
3 3
2 3
2 2
1 2
 */