from math import copysign, hypot
from typing import List


class point:
    __slots__ = ("x", "y")

    def __init__(self, x, y):
        self.x = x
        self.y = y

    def __sub__(self, other):
        return point(self.x - other.x, self.y - other.y)

    def angle(self):
        if self.x == 0:
            return copysign(float("inf"), self.y)
        else:
            return self.y / self.x

    def __repr__(self):
        return f"(x={self.x}, y={self.y})"

    #
    # def __hash__(self):
    #     return hash(complex(self.x, self.y))

    def __eq__(self, other):
        return self.x == other.x and self.y == other.y

    def __abs__(self):
        return hypot(self.x, self.y)


def wrapping(points: List[point]):
    points.sort(key=lambda p: (p.x, p.y))

    lh = 2
    history_top = points[:2]
    for p in points[2:]:
        history_top.append(p)
        lh += 1
        while lh > 2 and (history_top[-1] - history_top[-2]).angle() > (
                history_top[-2] - history_top[-3]).angle():
            history_top.pop(-2)
            lh -= 1

    lh = 2
    points.sort(key=lambda p: (p.x, -p.y))
    history_bottom = points[:2]
    for p in points[2:]:
        history_bottom.append(p)
        lh += 1
        while lh > 2 and (history_bottom[-1] - history_bottom[-2]).angle() < (
                history_bottom[-2] - history_bottom[-3]).angle():
            history_bottom.pop(-2)
            lh -= 1
            
    history_bottom.reverse()
    return history_top + history_bottom[:-2]


n, l = map(int, input().split())

points = set()
for _ in range(n):
    x, y = map(int, input().split())
    points.add(point(x - l, y))
    points.add(point(x + l, y))
    points.add(point(x, y - l))
    points.add(point(x, y + l))

w = wrapping(list(points))
s = 0
for i in range(len(w)):
    s += abs(w[i] - w[i - 1])

print(s)
