#include <iostream>
#include <vector>
#include <algorithm>
#include <random>
#include <chrono>
#include <set>
#include <map>
#include <iomanip>

using namespace std;

//#define int long long

typedef long long ll;

struct pt {
    ll x, y;

    pt() {}

    pt(ll _x, ll _y) {
        x = _x, y = _y;
    }

    ll operator%(const pt &oth) const {
        return x * oth.x + y * oth.y;
    }

    ll operator*(const pt &oth) const {
        return x * oth.y - y * oth.x;
    }

    pt operator-(const pt &oth) const {
        return pt(x - oth.x, y - oth.y);
    }

    bool operator<(const pt &oth) {
        if (y != oth.y) return y < oth.y;
        return x < oth.x;
    }
};

bool cmp(const pt &a, const pt &b) {
    return (a * b) > 0LL;
}

typedef long double ld;

ld getAngle(const pt &a, const pt &b) {
    return atan2(a * b, a % b);
}

ld getLength(const pt &a) {
    return sqrtl((ld) (a.x * a.x + a.y * a.y));
}

vector<pt> ch(vector<pt> &a) {
    int n = a.size();
    pt q = a[0];

    for (int i = 0; i < n; i++) {
        if (a[i] < q) q = a[i];
    }

    for (int i = 0; i < n; i++) {
        a[i] = a[i] - q;
    }

    sort(begin(a), end(a), cmp);

    int m = 0;
    vector<pt> ans;
    for (int i = 0; i < n; i++) {
        while (m >= 2 && ((ans[m - 1] - ans[m - 2]) * (a[i] - ans[m - 2])) <= 0LL) {
            ans.pop_back();
            m--;
        }
        ans.emplace_back(a[i]);
        m++;
    }

    return ans;
}

void solve() {
    ld pi = acosl(-1);

    int n, k;
    cin >> n >> k;
    vector<pt> a(n);
    for (int i = 0; i < n; i++) {
        cin >> a[i].x >> a[i].y;
    }
    a = ch(a);

    n = a.size();
//    for (int i = 0; i < n; i++){
//        cout << a[i].x << ' ' << a[i].y << '\n';
//    }
    ld res = 0;
    for (int i = 0; i < n; ++i) {
        int ii = (i + 1) % n;
        int iii = (i + 2) % n;

        pt p1 = a[i];
        pt p2 = a[ii];
        pt p3 = a[iii];

        res += getLength(p1 - p2);

        pt v1 = p1 - p2;
        pt v2 = p3 - p2;
        ld angle = pi - abs(getAngle(v1, v2));
        res += 2.0L * ((ld) k * sinl(angle / 2));
    }

    cout << setprecision(239) << res;
}

signed main() {
    ios_base::sync_with_stdio(0);
    cin.tie(0);
    cout.tie(0);

    solve();

    return 0;
}