#include <bits/stdc++.h>
using namespace std;

struct operation {
    complex<int> rotation;
    complex<int> shift_vector;

    complex<int> apply(complex<int> x) {
        return x * rotation + shift_vector;
    }

    operation(): rotation(1), shift_vector(0) {}
    operation(complex<int> r, complex<int> s): rotation(r), shift_vector(s) {}

    static operation get_none() {
        return operation();
    }

    static operation get_rotation(complex<int> center, complex<int> rotation) {
        return {rotation, -center * rotation + center};
    }

    static operation get_counterclockwise(complex<int> center) {
        return get_rotation(center, {0, 1});
    }

    static operation get_clockwise(complex<int> center) {
        return get_rotation(center, {0, -1});
    }
};

operation composition(operation first, operation second) {
    operation new_op = {first.rotation * second.rotation,
                        first.shift_vector * second.rotation + second.shift_vector};
    return new_op;
}

const int MAX_N = 2e5;
complex<int> a[MAX_N];
complex<int> t[4 * MAX_N];
operation f[4 * MAX_N];

void build(int v, int tl, int tr) {
    if (tl == tr - 1) {
        t[v] = a[tl];
    }
    else {
        int tm = (tl + tr) / 2;
        build(2 * v + 1, tl, tm);
        build(2 * v + 2, tm, tr);
    }
}

void apply(int v, int tl, int tr, operation op) {
    if(tl == tr - 1) {
        t[v] = op.apply(t[v]);
    }
    else {
        f[v] = composition(f[v], op);
    }
}

void push(int v, int tl, int tr) {
    int tm = (tl + tr) / 2;
    apply(2 * v + 1, tl, tm, f[v]);
    apply(2 * v + 2, tm, tr, f[v]);
    f[v] = operation::get_none();
}

void update(int v, int tl, int tr, int l, int r, operation op) {
    if (l >= r){
        return;
    }
    else if (l == tl && r == tr) {
        apply(v, tl, tr, op);
    }
    else {
        push(v, tl, tr);
        int tm = (tl + tr) / 2;
        update(2 * v + 1, tl, tm, l, min(r, tm), op);
        update(2 * v + 2, tm, tr, max(l, tm), r, op);
    }
}

complex<int> get(int v, int tl, int tr, int pos) {
    if (tl == tr - 1)
        return t[v];
    else {
        push(v, tl, tr);
        int tm = (tl + tr) / 2;
        if (pos < tm)
            return get(2 * v + 1, tl, tm, pos);
        else
            return get(2 * v + 2, tm, tr, pos);
    }
}


int main(){
    ios_base::sync_with_stdio(0); cin.tie(0); cout.tie(0);

    int n, q;
    cin >> n >> q;
    for (int i = 0; i < n; i++)
        a[i] = {i, 0};
    build(0, 0, n);

    for (int i = 0; i < q; i++) {
        int type;
        cin >> type;
        if (type == 1) {
            int l, r;
            cin >> l >> r;
            l--;
            if (r - l == 1)
                continue;
            complex<int> f = get(0, 0, n, l);
            complex<int> s = get(0, 0, n, l + 1);
            operation op;
            if(f.real() == s.real()) {  /// x is same, this is vertical
                op = operation::get_clockwise(f);
            }
            else if (f.imag() == s.imag()) {
                op = operation::get_counterclockwise(f);
            }
            else
                assert(false);
            update(0, 0, n, l, r, op);
        }
        else {
            int j;
            cin >> j;
            j--;
            complex<int> pos = get(0, 0, n, j);
            cout << pos.real() + 1 << " " << pos.imag() << "\n";
        }
    }

    return 0;
}
