#include <iostream>
#include <cmath>
#include <algorithm>
#include <string>
#include <vector>
#include <set>
#include <unordered_set>
#include <climits>
#include <map>
#include <iomanip>
#include <queue>
#include <random>
#include <ctime>

using namespace std;

#pragma warning (disable: 4996)

const long double EPS = 1e-9;

struct Point 
{
    int x, y;
    Point() {};
    Point(int x, int y) : x(x), y(y) {};
    bool operator < (Point p1)
    {
        if (x != p1.x)
            return x < p1.x;
        return y < p1.y;
    }

    Point operator- (Point p1)
    {
        return Point(x - p1.x, y - p1.y);
    }
};

int operator* (Point a, Point b) {
    return a.x * b.x + a.y * b.y;
}

int operator^ (Point a, Point b) {
    return a.x * b.y - a.y * b.x;
}

bool Less(long double d1, long double d2)
{
    return d1 + EPS < d2;
}

long double dist(Point& p1, Point& p2, long long add = 0)
{
    return sqrt((abs(p1.x - p2.x) + add) * (abs(p1.x - p2.x) + add) + (abs(p1.y - p2.y) + add) * (abs(p1.y - p2.y) + add));
}

void getConvexHull(vector<Point>& p, vector<Point>& cH)
{
    vector<Point> cHUp, cHDown;
    sort(p.begin(), p.end());
    int n = p.size();
    for (int i = 0; i < n; i++)
    {
        while (cHUp.size() > 1 && ((p[i] - cHUp.back()) ^ (cHUp.back() - cHUp[cHUp.size() - 2])) > 0)
            cHUp.pop_back();

        while (cHDown.size() > 1 && ((p[i] - cHDown.back()) ^ (cHDown.back() - cHDown[cHDown.size() - 2])) < 0)
            cHDown.pop_back();

        cHDown.push_back(p[i]);
        cHUp.push_back(p[i]);
    }
    for (Point e : cHUp)
        cH.push_back(e);
    for (int i = cHDown.size() - 2; i > 0; i--)
        cH.push_back(cHDown[i]);
}

int main() {
    ios_base::sync_with_stdio(false);
    cin.tie(0);
    cout.tie(0);
    srand(time(0));
#ifdef _DEBUG
    freopen("input.txt", "r", stdin);
#endif
    int n, l;
    cin >> n >> l;
    vector<Point> inp(n), pol;
    for (int i = 0; i < n; i++)
        cin >> inp[i].x >> inp[i].y;
    getConvexHull(inp, pol);
    long double ans = 0;
    n = pol.size();
    for (int i = 0; i < n; i++)
    {
        Point next = pol[(i + 1) % n];
        if (pol[i].x == next.x || pol[i].y == next.y)
            ans += dist(pol[i], next);
        else
            ans += dist(pol[i], next, l);
    }

    for (int i = 0; i < n; i++)
    {
        Point prev = pol[(i - 1 + n) % n], next = pol[(i + 1) % n];
        if (((pol[i] - prev) * (next - pol[i])) == 0)
            ans += l * sqrt(2);
    }
    cout << fixed << setprecision(10) << ans;
}